package jalview.controller;

import static org.testng.AssertJUnit.assertEquals;
import static org.testng.AssertJUnit.assertTrue;

import jalview.datamodel.Sequence;
import jalview.datamodel.SequenceFeature;
import jalview.datamodel.SequenceGroup;
import jalview.datamodel.SequenceI;

import java.util.BitSet;

import org.testng.annotations.Test;

public class AlignViewControllerTest
{
  @Test(groups = "Functional")
  public void testFindColumnsWithFeature()
  {
    SequenceI seq1 = new Sequence("seq1", "aMMMaaaaaaaaaaaaaaaa");
    SequenceI seq2 = new Sequence("seq2", "aaaMMMMMMMaaaaaaaaaa");
    SequenceI seq3 = new Sequence("seq3", "aaaaaaaaaaMMMMMaaaaa");
    SequenceI seq4 = new Sequence("seq3", "aaaaaaaaaaaaaaaaaaaa");

    /*
     * features start/end are base 1
     */
    seq1.addSequenceFeature(new SequenceFeature("Metal", "desc", 2, 4, 0f,
            null));
    seq1.addSequenceFeature(new SequenceFeature("Helix", "desc", 1, 15, 0f,
            null));
    seq2.addSequenceFeature(new SequenceFeature("Metal", "desc", 4, 10, 0f,
            null));
    seq3.addSequenceFeature(new SequenceFeature("Metal", "desc", 11, 15,
            0f, null));

    /*
     * select the first three columns --> Metal in seq1 2-3
     */
    SequenceGroup sg = new SequenceGroup();
    sg.setStartRes(0); // base 0
    sg.setEndRes(2);
    sg.addSequence(seq1, false);
    sg.addSequence(seq2, false);
    sg.addSequence(seq3, false);
    sg.addSequence(seq4, false);

    BitSet bs = new BitSet();
    int seqCount = AlignViewController.findColumnsWithFeature("Metal", sg,
            bs);
    assertEquals(1, seqCount);
    assertEquals(2, bs.cardinality());
    assertTrue(bs.get(1));
    assertTrue(bs.get(2));
    
    /*
     * select the first four columns: Metal in seq1 2:4, seq2 4:4
     */
    sg.setEndRes(3);
    bs.clear();
    seqCount = AlignViewController.findColumnsWithFeature("Metal", sg,
            bs);
    assertEquals(2, seqCount);
    assertEquals(3, bs.cardinality());
    assertTrue(bs.get(1));
    assertTrue(bs.get(2));
    assertTrue(bs.get(3));

    /*
     * select column 11: Metal in seq3 only
     */
    sg.setStartRes(10);
    sg.setEndRes(10);
    bs.clear();
    seqCount = AlignViewController.findColumnsWithFeature("Metal", sg, bs);
    assertEquals(1, seqCount);
    assertEquals(1, bs.cardinality());
    assertTrue(bs.get(10));

    /*
     * select columns 16-20: no Metal feature
     */
    sg.setStartRes(15);
    sg.setEndRes(19);
    bs.clear();
    seqCount = AlignViewController.findColumnsWithFeature("Metal", sg, bs);
    assertEquals(0, seqCount);
    assertEquals(0, bs.cardinality());

    /*
     * look for a feature that isn't there
     */
    sg.setStartRes(0);
    sg.setEndRes(19);
    bs.clear();
    seqCount = AlignViewController.findColumnsWithFeature("Pfam", sg, bs);
    assertEquals(0, seqCount);
    assertEquals(0, bs.cardinality());
  }
}
